require "raycast"

pixels = {}

drawRays = false

planeResX = 256
planeResY = 256
planeScale = 2
padding = 200
rayDist = 1000
rotation = 0
rotSpeed = 80
moveSpeed = 50

planeData = {}
planeImageData = love.image.newImageData(planeResX*planeScale, planeResY*planeScale)
planeImage = love.graphics.newImage(planeImageData)

cameraX = 0
cameraY = 0
fov = 60
fovSpeed = 40
camScale = 30
lineRes = 1024
lineScale = 0.75
maxScale = 15
minDist = 15
lineData = {}
lineImageData = love.image.newImageData(lineRes*lineScale, lineRes*lineScale)

planeX = lineScale + padding
planeY = (love.graphics.getHeight() - planeResY*planeScale) / 2

lineX = (love.graphics.getWidth() - lineRes*lineScale) / 2

raycast.setVars(planeX, planeY, planeResX, planeResY, planeScale)

pixels.load = function()
	--randomly initialize plane
	for x=1, planeResX do
		planeData[x] = {}
		for y=1, planeResY do
			r = math.random()*255
			g = math.random()*255
			b = math.random()*255
			a = math.random(100, 255)
			if math.random() < 0.98 then
				planeData[x][y] = nil
			elseif math.random() < 0.4 then
				planeData[x][y] = {r, g, b, 255}
			else
				planeData[x][y] = {r, g, b, a}
			end
			for x2=0, planeScale-1 do
				for y2=0, planeScale-1 do
					if planeData[x][y] then
						planeImageData:setPixel(planeScale*(x-1)+x2, planeScale*(y-1)+y2, planeData[x][y])
					else
						planeImageData:setPixel(planeScale*(x-1)+x2, planeScale*(y-1)+y2, 255, 255, 255, 255)
					end
				end
			end
		end
	end
	planeImage = love.graphics.newImage(planeImageData)
	
	cameraX = love.graphics.getWidth() / 2
	cameraY = love.graphics.getHeight() / 2
end

pixels.rotateCam = function(dt, r)
	rotation = (rotation + (r * dt * rotSpeed)) % 360
end

pixels.moveCam = function(dt, xDir, yDir)
	dirY = {math.sin(math.rad(rotation)), -math.cos(math.rad(rotation))}
	dirX = {math.cos(math.rad(rotation)), math.sin(math.rad(rotation))}
	
	cameraX = cameraX + (-yDir * (dirY[1] * dt * moveSpeed))
	cameraY = cameraY + (-yDir * (dirY[2] * dt * moveSpeed))
	
	cameraX = cameraX + (xDir * (dirX[1] * dt * moveSpeed))
	cameraY = cameraY + (xDir * (dirX[2] * dt * moveSpeed))
end

pixels.changeFov = function(dt, dir)
	fov = fov + (dir * dt * fovSpeed)
end

pixels.update = function(dt)
	if love.keyboard.isDown("w") then
		pixels.moveCam(dt, 0, -1)
	elseif love.keyboard.isDown("s") then
		pixels.moveCam(dt, 0, 1)
	end
	if love.keyboard.isDown("a") then
		pixels.rotateCam(dt, -1)
	elseif love.keyboard.isDown("d") then
		pixels.rotateCam(dt, 1)
	end
	
	if love.keyboard.isDown("lshift") then
		rotSpeed = 160
		moveSpeed = 100
	elseif love.keyboard.isDown("lctrl") then
		rotSpeed = 40
		moveSpeed = 20
	else
		rotSpeed = 80
		moveSpeed = 50
	end
	
	-- keep inside bounds
	if cameraX - planeX < 0 then
		cameraX = 0 + planeX
	elseif cameraX - planeX > planeResX*planeScale - 1 then
		cameraX = planeResX*planeScale - 1 + planeX
	end
	if cameraY - planeY < 0 then
		cameraY = 0 + planeY
	elseif cameraY - planeY > planeResY*planeScale - 1 then
		cameraY = planeResY*planeScale - 1 + planeY
	end
	
	-- get line data
	checkedPixels = {}
	for i=1, lineRes do
		lineData[i] = raycast.cast(planeData, cameraX, cameraY, rotation + (fov/lineRes * i) - (fov/2) - (fov/lineRes / 2))
	end
end

pixels.drawLine = function()
	for i=1, lineRes do
		if lineData[i].color then
			love.graphics.setColor(lineData[i].color)
		else
			love.graphics.setColor(255, 255, 255, 255)
		end
		scaleY = maxScale - (lineData[i].dist / minDist)
		if scaleY < 0 then
			scaleY = 0
		end
		love.graphics.rectangle("fill", (i-1)*lineScale + lineX, love.graphics.getHeight() - scaleY - 10, lineScale, scaleY)
	end
end

pixels.drawCamera = function()
	love.graphics.translate(cameraX, cameraY)
	love.graphics.rotate(math.rad(rotation))
	love.graphics.translate(-cameraX, -cameraY)
	
	--rays
	if drawRays then
		for i=1, lineRes do
			--center
			rot = (fov/lineRes)*i - (fov/2) - (fov/lineRes / 2)
			xoff = math.tan(math.rad(rot)) * rayDist
			love.graphics.setColor(255, 0, 0, 100)
			love.graphics.setLineWidth(1)
			love.graphics.line(cameraX, cameraY, cameraX + xoff, cameraY - rayDist)
			--bounds
			rot = rot - (fov/lineRes / 2)
			xoff = math.tan(math.rad(rot)) * rayDist
			love.graphics.setColor(0, 255, 0, 50)
			love.graphics.setLineWidth(0.5)
			love.graphics.line(cameraX, cameraY, cameraX + xoff, cameraY - rayDist)
		end
		rot = (fov/lineRes)*lineRes - (fov/2)
		xoff = math.tan(math.rad(rot)) * rayDist
		love.graphics.line(cameraX, cameraY, cameraX + xoff, cameraY - rayDist)
	end
	
	
	--camera
	love.graphics.setColor(0, 0, 255, 180)
	love.graphics.setLineWidth(1.5)
	xoff = math.tan(math.rad(fov)/2) * camScale
	love.graphics.polygon("line", cameraX, cameraY, cameraX - xoff, cameraY - camScale, cameraX + xoff, cameraY - camScale)
	love.graphics.setPointSize(5)
	love.graphics.point(cameraX, cameraY)
	
	love.graphics.translate(cameraX, cameraY)
	love.graphics.rotate(-math.rad(rotation))
	love.graphics.translate(-cameraX, -cameraY)
end

pixels.draw = function()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(planeImage, planeX, planeY)
	pixels.drawLine()
	pixels.drawCamera()
	--debug
	--raycast.drawDebug()
end