-- reference: http://www.cse.yorku.ca/~amana/research/grid.pdf

raycast = {}

raycast.setVars = function(planeX, planeY, planeResX, planeResY, planeScale)
	planeX = planeX
	planeY = planeY
	planeResX = planeResX
	planeResY = planeResY
	planeScale = planeScale
end

bgCol = {240, 240, 240, 255}

-- pre-set some variables for debugging
dir = {0, 0}
stepX = 0
stepY = 0
originX = 0
originY = 0
tMaxX = 0
tMaxY = 0
tDeltaX = 0
tDeltaY = 0

checkedPixels = {{x=2, y=2}, {x=50, y=50}}

raycast.cast = function(plane, camX, camY, rot)
	-- convert to vector
	dir = {math.sin(math.rad(rot)), -math.cos(math.rad(rot))}
	
	-- find starting point
	if camX-planeX >= 0 and camX-planeX < planeResX*planeScale and
	camY-planeY >= 0 and camY-planeY < planeResY*planeScale then
		startX = math.floor((camX-planeX)/planeScale) + 1
		startY = math.floor((camY-planeY)/planeScale) + 1
		X = startX
		Y = startY
		originX = camX-planeX
		originY = camY-planeY
	else
		if raycast.findStart(plane, camX, camY, rot) == nil then --if nil return background
			return bgCol
		else
			X, Y, originX, originY = raycast.findStart(plane, camX, camY, rot) --if not nil set it
		end
	end
	
	-- step direction
	if dir[1] > 0 then
		stepX = 1
	else
		stepX = -1
	end
	if dir[2] > 0 then
		stepY = 1
	else
		stepY = -1
	end
	
	tMaxX = math.abs(1 - (camX - math.floor(camX)) / planeScale) / dir[1]
	tMaxY = math.abs(1 - (camY - math.floor(camY)) / planeScale) / dir[2]
	tDeltaX = 1 / dir[1]
	tDeltaY = 1 / dir[2]
	
	xMin = 0
	xMax = planeResX
	yMin = 0
	yMax = planeResY
	
	-- for alpha blending (need to add backwards after collection)
	rVals = {}
	gVals = {}
	bVals = {}
	aVals = {}
	
	firstX = nil
	firstY = nil
	curAlpha = 0
	while X >= xMin and X <= xMax and Y >= yMin and Y <= yMax and curAlpha ~= 255 do
		if plane[X] and plane[X][Y] then
			table.insert(rVals, 1, plane[X][Y][1])
			table.insert(gVals, 1, plane[X][Y][2])
			table.insert(bVals, 1, plane[X][Y][3])
			table.insert(aVals, 1, plane[X][Y][4])
			
			table.insert(checkedPixels, {x=X-1, y=Y-1})
			
			if firstX == nil and firstY == nil then
				firstX = X
				firstY = Y
			end
		end
		
		if tMaxX < tMaxY then
			tMaxX = tMaxX + (tDeltaX * stepX)
			X = X + stepX
		else
			tMaxY = tMaxY + (tDeltaY * stepY)
			Y = Y + stepY
		end
	end
	
	-- alpha blending currently unimplemented
	r = bgCol[1]
	g = bgCol[2]
	b = bgCol[3]
	a = bgCol[4]
	if rVals[#rVals] and gVals[#gVals] and bVals[#bVals] and aVals[#aVals] then --if empty use background
		r = rVals[#rVals]
		g = gVals[#gVals]
		b = bVals[#bVals]
		a = aVals[#aVals]
	end
	
	if firstX == nil and firstY == nil then
		dist = 1/0 -- infinity
	else
		dist = math.sqrt(math.abs(startX - firstX)^2 + math.abs(startY - firstY)^2)
	end
	return {color = {r, g, b, a}, dist = dist}
end

raycast.findStart = function(plane, camX, camY, rot)
	-- actually find it if you can
	
	return nil
end

raycast.drawDebug = function()
	love.graphics.setColor(0, 0, 0)
	love.graphics.print("Direction Vector: ("..dir[1]..", "..dir[2]..")", 200, 10)
	love.graphics.print("Step Values: ("..stepX..", "..stepY..")", 200, 25)
	love.graphics.print("t Vars: (tMaxX = "..tMaxX..", tMaxY = "..tMaxY..",\ntDeltaX = "..tDeltaX..", tDeltaY = "..tDeltaY..")", 300, 570)
	
	love.graphics.setColor(0, 255, 255, 255)
	love.graphics.setLineWidth(2)
	for i, v in pairs(checkedPixels) do
		love.graphics.rectangle("line", planeX+(v.x*planeScale), planeY+(v.y*planeScale), planeScale, planeScale)
	end
end
